//
// uses simpson to calculate the upper confidence point
// of the integral of the Fisher F-distribution function
// with k and n-k degrees of freedom such that the
// integral is equal to conf.  This upper limit is the
// factor ff used in the nllsq and fit subroutines.
//

#include "fit.h"

dbl fisher(int k, int n, dbl conf)
{
    dbl f, sum, h, a, b, y, yold, y0;
    
    a = normf(k, n) / 3.0;  // normf is the normalization factor
    y0 = conf;              // y0 is desired area under curve
    h = 0.005;              // h is simpson delta-f
    
    for (f = 0.0, y = 0.0 ; ; f += 2.0 * h) {
        sum = fish( f, k, n );
        sum += 4.0 * fish(f + h, k, n);
        sum += fish(f + 2.0 * h, k, n);
        yold = y;
        y += a * h * sum;   // y = normalized integral
                            // of fish(f)df from 0 to 2mh
    
// interpolation for last twiddle

        if (yold < y0 && y >= y0) { 
            b = 2.0 * h * (y0 - yold)/(y - yold);
            return(f + b);
        }
        else if (yold > y0)
            err( "couldn't find Fisher F-factor." );
    }
}

