//
// makes points for x and f(x) and prints them in
// output file ("datafile.out").  there is a nice
// algorithm for doing this.  there may be problems
// if the slope of the curve becomes infinite.
//


#include "fit.h"

cdbl    epsx = 0.002;
cdbl    epsy = 0.005;
cint    maxpts = 30000;                   // no curve should need this many pts

void func_pts(dbl* x, dbl* y, dbl* b, int npts, misc* mp)
{
    int i, j;
    dbl x_range, y_range, x0, x1;         // x0, x1 used for storage

    mp->lm = new dbl [4];                 // xmin, xmax, ymin, ymax
    dbl dx, f, df, t2;
    dbl dfdx;

    mp->lm[0] = mp->lm[1] = x[0];         // arbitrarily set to first pts
    mp->lm[2] = mp->lm[3] = y[0];

    for ( i = 0 ; i < npts ; i++ ) {      // determine limits of data
        mp->lm[0] = ( mp->lm[0] > x[i] ) ? x[i] : mp->lm[0];
        mp->lm[1] = ( mp->lm[1] < x[i] ) ? x[i] : mp->lm[1];
        mp->lm[2] = ( mp->lm[2] > y[i] ) ? y[i] : mp->lm[2];
        mp->lm[3] = ( mp->lm[3] < y[i] ) ? y[i] : mp->lm[3];
    }
    x_range = mp->lm[1] - mp->lm[0];
    y_range = mp->lm[3] - mp->lm[2];

    x0 = x[0];                            // store x[0] and x[1]
    x1 = x[1];

    dx = epsx * x_range;                  // initial dx
    t2 = epsy * y_range;                  // for testing df
    x[0] = mp->lm[0] - mp->fract * x_range;   // initial x[0] < xmin
    f = func(x, b, 0, &dfdx, 0, 0, 1);    // f(x[0])

    j = mp->prec + 10;                    // used for field width
    //
    // we print points to datafile.out
    //
    gassert(mp->ofs);
    mp->ofs.precision(mp->prec+1);        // # of sig figs in output
    mp->ofs.fill(' ');                    // fills field with spaces
    mp->ofs.setf(ios::left);              // #s at left of field
    mp->ofs.setf(ios::floatfield);        // choses either fixed or scientific
    mp->ofs.setf(ios::showpoint);         // ensures decimal point

    mp->ofs << setw(j) << x[0] << setw(j) << f << "\n";  // 1st point

    mp->lm[0] = x[0];                     // set new xmin
    mp->lm[1] += mp->fract * x_range;     // extend xmax by fract
    x[1] = x[0] + dx;                     // initial x[1]
    df = del_f(x, b);                     // initial df

    i = 0;                                // i will count steps
    while (x[0] < mp->lm[1]) {
        if (++i > maxpts) {
            err( "\nencountered an error in calculating the points\n"
                 "for the theoretical curve fit to the data.\n"
                 "this may arise from either an infinite slope\n"
                 "in this curve, or in trying to calculate points\n"
                 "in a forbidden region of the curve, e.g. at a\n"
                 "singularity.  you might try reducing \'extend\' to\n"
                 "zero.  you may otherwise check to see that df/dx is\n"
                 "correctly calculated in your code for func.c.\n\n"
                 "to see where it broke down, have a look at\n"
                 "\'", mp->ofile[0].c_str(), "\'.\n"); // (datafile.out)
        }
        while (fabs( df ) < t2) {        // increase dx until
            if (x[1] == mp->lm[1])       // df reaches t2
                break;                   // or x[1] = xmax
            dx *= 2.0;
            x[1] = x[0] + dx;
            x[1] = (x[1] > mp->lm[1]) ? mp->lm[1] : x[1];
            df = del_f(x, b);
        }
    
        while (fabs( df) >= t2) {        // decrease dx until
            dx /= 2.0;                   // df is just < t2
            x[1] = x[0] + dx;
            df = del_f(x, b);
        }
        f = func(x, b, 0, &dfdx, 0, 1, 1);   // f(x[1])
        mp->ofs << setw(j) << x[1] << setw(j) << f << "\n"; // succeeding pts
        x[0] = x[1];
        dx = epsx * x_range;             // reset dx for next loop
    }
    x[0] = x0;                           // restore the x-vector when done
    x[1] = x1;
}


