//
// gets options from the command line,
// and sets various flags.
//

#include "fit.h"
// #include "getopt.h"

void get_opts(int argc, char** argv, flags* fp, misc* mp)
{
    extern int opterr;
    extern char *optarg;
    int i;
    mp->fract = 0.05;             // default value of fract

#ifdef __GLIBC__
    int opt_index = 0;
    static struct option long_opts[] = {
        {"help", 0, 0, 'h'},
        {"version", 0, 0, 'v'},
        {"extend", 0, 0, 'e'},
        {"output", 1, 0, 'o'},
        {"precision", 1, 0, 'p'},
        {"quiet", 0, 0, 'q'},
        {"sigma", 0, 0, 's'},
        {0, 0, 0, 0}
    };
#endif

    opterr = on;

#ifdef __GLIBC__
    while ((i = getopt_long( argc, argv, "heo:p:qsv",
            long_opts, &opt_index)) != -1 ) {
#else                            // for SunOs machines
    while ((i = getopt(argc, argv, "heopqsv")) != -1){
#endif
        switch (i) {
            case 'h':            // display help file
                prn_info(argv);
                exit(0);
                break;
            case 'e':
                dbl extend;
                cout << "\nextend curve range by what percent? " ;
                cin >> extend;
                mp->fract = 0.005 * extend;
                break;
            case 'o':            // make output file for plotting
                if (!strncmp(optarg, "goplot", 3)) {
                    fp->o = goplot;
                }
                else if (!strncmp(optarg, "gnuplot", 5)) {
                    fp->o = gnuplot;
                }
                else if (!strncmp( optarg, "gnups", 5)) {
                    fp->o = gnups;
                }
                else {
                    prn_usage(argv);
                    exit(1);
                }
                break;
            case 'p':            // specify output significant figs
                fp->p = on;
                mp->prec = atoi(optarg);
                break;
            case 'q':             // suppress intermediate output
                fp->q = on;
                break;
            case 's':             // if "absolute" sigmas supplied
                fp->s = on;
                break;
            case 'v':             // print version number
                cerr << argv[0] << " version " << version << "\n";
                exit(0);
                break;
            case '?':             // for unrecognized option
                prn_usage( argv );
                exit( 1 );
        }
    }
	if (!argv[optind])
		err("did you forget the datafile?");
	if (!argv[optind+1])
		err("did you forget the paramfile?");
    if (fp->o == goplot) {
        mp->ofile[0] = argv[optind];
        mp->ofile[0] += ".out";
    }
    if (fp->o == gnuplot || fp->o == gnups) {
        mp->ofile[0] = argv[optind];
        mp->ofile[0] += ".out";
        mp->ofile[1] = argv[optind];
        mp->ofile[1] += ".gnu";
    }
    if (!fp->p) mp->prec = 3;  // default precision for output

    if (mp->prec < 1) {
        err("precision must be a positive integer");
    }
}
