//
// creates the files "func.c", "Makefile", "gaussdata" and
// "gausspars" for testing and revising the fit program.
//

#include "mff.h"

void mk_funcfile(void)
{
    string ofn = "./func.c";
    ofstream ofs;
    ofs.open(ofn.c_str());            // opens "func.c"
    gassert(ofs);
    ofs << "// \n"
           "// this is \"func.c\", the single function that must be\n"
           "// supplied by the user in the \"fit\" program.\n"
           "// \n"
           "// it contains the form of the function to be fit to\n"
           "// the data, along with its derivatives with respect\n"
           "// to both the parameters b[j] and the independent\n"
           "// variable, here called \"x\".\n"
           "// \n"
           "// see the man page for \"fit\" (\"man fit\") for hints\n"
           "// on how to write the code in a compact manner.\n"
           "// \n"
           "\n"
           "#include \"fit.h\"\n"
           "\n"
           "dbl func(dbl* x, dbl* b, dbl* p, dbl* dfdx, misc* mp, int i, int j)\n"
           "{\n"
           "\n"
           "// i numbers the point; j is used in switch below\n"
           "// f is the function to be fit to the data\n"
           "\n"
           "    dbl f = 0.0;\n"
           "\n"
           "    if (j) {         \n"
           "        f  = b[0] * exp( - (x[i]-b[1])*(x[i]-b[1])/(2.0*b[2]*b[2]));\n"
           "        f += b[3]; // this fits a gaussian plus const background\n"
           "    }\n"
           "\n"
           "    switch (j) {\n"
           "        case 0:  // members of the structure pointed to by mp:\n"
           "\n"
           "           // title: not more than 4 or 5 words describing function\n"
           "            mp->title = \"a single gaussian line\";\n"
           "\n"
           "           // npar: the number of parameters\n"
           "            mp->npar = 4;\n"
           "\n"
           "           // maxiter: maximum number of iterations\n"
           "            mp->maxiter = 20;     // default is MAXITER = 30\n"
           "\n"
           "           // lambda: starting value of Marquardt parameter\n"
           "            mp->lambda = 5.0e-6;  // default is LAMBDA = 1.0e-8\n"
           "\n"
           "           // names of parameters to be used for the b[k]\n"
           "            mp->name[0] = \"A\";\n"
           "            mp->name[1] = \"mu\";\n"
           "            mp->name[2] = \"width\";\n"
           "            mp->name[3] = \"base\";\n"
           "\n"
           "           // confidence level used to calculate uncertainties\n"
           "            mp->conf = CONF;      // default is CONF = .68269\n"
           "            break;\n"
           "\n"
           "        case 1:          // just f is returned\n"
           "            break;\n"
           "\n"
           "        case 2:          // p[k] is df/db[k]\n"
           "            p[0] =  ( f - b[3] ) / b[0];\n"
           "            p[1] =  ( f - b[3] ) * (x[i]-b[1]) / ( b[2] * b[2] );\n"
           "            p[2] =  ( f - b[3] ) * (x[i]-b[1]) *(x[i]-b[1]);\n"
           "            p[2] /=  b[2]*b[2]*b[2];\n"
           "            p[3] =  1.0;\n"
           "            break;\n"
           "\n"
           "        case 3:          // df/dx used in graphing f(x)\n"
           "            *dfdx = - (f-b[3]) * (x[i]-b[1]) / (b[2]*b[2]);\n"
           "            break;\n"
           "    }\n"
           "    return f;\n"
           "}\n";
    ofs.close();
    ofn = "Makefile";
    ofs.open(ofn.c_str());           // opens "Makefile"
    gassert(ofs);
    ofs << "# This is a makefile that may be used for compiling the\n"
           "# user-supplied function \"func.c\" with the already-compiled\n"
           "# library of fit functions (\"libfit.a\") to produce a working\n"
           "# curve-fitting program.\n"
           "# \n"
           "# This makefile may need to be edited to reflect correctly\n"
           "# the locations of the fit program header files and library\n"
           "# of functions.  For example, if the header files have been\n"
           "# installed in /usr/local/include and the library in\n"
           "# /usr/local/lib, then the first PREFIX line below should be\n"
           "# commented out or deleted, and the second PREFIX line should\n"
           "# be uncommented.\n"
           "#\n"
           "# After the makefile is properly edited just give the\n"
           "# command \"make\" and if the already-compiled fit library\n"
           "# (\"libfit.a\") and the appropriate header files (\"fit.h\")\n"
           "# are installed and readable in the directories indicated by\n"
           "# the \"LIBS\" and \"INCLS\" variables below, everything should\n"
           "# work.  The resulting executable (defined by the \"EXEC\"\n"
           "# variable below [the default name is \"fit\"]) should appear\n"
           "# in the current working directory unless this file is edited\n"
           "# so as to change the \"EXEC\" variable.\n"
           "#\n"
           "# The command \"make clean\" will remove the executable, the\n"
           "# object files, and any files with the extensions \".out\",\n"
           "# \".gnu\" or \".eps\" that may have been produced.\n"
           "#\n"
           "\n"
           "PREFIX  = ..\n"
           "# PREFIX = /usr/local\n"
           "CC      =  g++\n"
           "CFLAGS  =  -Wall -s\n"
           "\n"
           "# the next line says to put the executable in the current directory\n"
           "\n"
           "EXEC    =  ./fit \n"
           "INCLS   =  -I$(PREFIX)/include\n"
           "LIBS    =  $(PREFIX)/lib/libfit.a\n"
           "\n"
           "OBJS    =  func.o\n"
           "\n"
           "$(EXEC): $(OBJS)\n"
           "\t@echo \"linking ...\"\n"
           "\t@$(CC)  $(CFLAGS)  -o $(EXEC)  $(OBJS)  $(LIBS)\n"
           "\n"
           "%.o : %.c\n"
           "\t$(CC)  $(CFLAGS)  $(INCLS)  -c  $<\n"
           "\n"
           "clean:\n"
           "\trm -f *.o $(EXEC) *.out *.gnu *.eps\n";
    ofs.close();
    ofn = "gaussdata";
    ofs.open(ofn.c_str());
    gassert(ofs);
    ofs << "#\n"
           "#-----------------------------------------\n"
           "#data for a single gaussian lineshape from\n"
           "#   the X-ray fluorescence experiment\n"
           "#    CHAN#  COUNTS SIGMA\n"
           "#-----------------------------------------\n"
           "       9     490   22.14\n"
           "      10     517   22.74\n"
           "      11     530   23.02\n"
           "      12     567   23.81\n"
           "      13     642   25.34\n"
           "      14     650   25.50\n"
           "      15     766   27.68\n"
           "      16     844   29.05\n"
           "      17     982   31.34\n"
           "      18    1112   33.35\n"
           "      19    1227   35.03\n"
           "      20    1356   36.82\n"
           "      21    1537   39.20\n"
           "      22    1658   40.72\n"
           "      23    1650   40.62\n"
           "      24    1699   41.22\n"
           "      25    1681   41.00\n"
           "      26    1621   40.26\n"
           "      27    1541   39.26\n"
           "      28    1438   37.92\n"
           "      29    1324   36.39\n"
           "      30    1173   34.25\n"
           "      31    1033   32.14\n"
           "      32     944   30.72\n"
           "      33     775   27.84\n"
           "      34     766   27.68\n"
           "      35     703   26.51\n"
           "      36     643   25.36\n"
           "      37     606   24.62\n"
           "      38     598   24.45\n"
           "      39     546   23.37\n"
           "      40     491   22.16\n";

    ofs.close();
    ofn = "gausspars";
    ofs.open(ofn.c_str());
    gassert(ofs);
    ofs << "2300.0 25.0\n"
           "5.0\n"
           "300.0\n";

    ofs.close();
}
